@@ -8,11 +8,11 @@ use OAR::IO;
 use OAR::Conf qw(init_conf dump_conf get_conf is_conf);
 
 my $base;
+my $current_date = -1;
 
 # Read config
 init_conf($ENV{OARCONFFILE});
 my $Cpuset_field = get_conf("JOB_RESOURCE_MANAGER_PROPERTY_DB_FIELD");
-my $Job_uid_resource_type = get_conf("JOB_RESOURCE_MANAGER_JOB_UID_TYPE");
 
 sub open_db_connection(){
 	$base  = OAR::IO::connect_ro_one();
@@ -21,7 +21,8 @@ sub open_db_connection(){
 }
 
 sub close_db_connection(){
-	OAR::IO::disconnect($base);
+	OAR::IO::disconnect($base) if (defined($base));
+    $base = undef;
 }
 
 sub get_oar_version(){
@@ -78,7 +79,7 @@ sub get_accounting_summary_byproject($$$){
     my $start = shift;
     my $stop = shift;
     my $user = shift;
-	return OAR::IO::get_accounting_summary_byproject($base,$start,$stop,$user);
+	return OAR::IO::get_accounting_summary_byproject($base,$start,$stop,$user,undef,undef);
 }
 
 sub get_array_job_ids($){
@@ -97,14 +98,9 @@ sub get_array_subjobs {
 
 sub get_all_jobs_for_user {
   my $user = shift;
-  my @jobs;
+
   my @states =  ("Finishing", "Running", "Resuming", "Suspended", "Launching", "toLaunch", "Waiting", "toAckReservation", "Hold");
-  my @get_jobs_in_state_for_user;
-  foreach my $current_state (@states){
-    @get_jobs_in_state_for_user = OAR::IO::get_jobs_in_state_for_user($base, $current_state, $user);
-    push( @jobs, @get_jobs_in_state_for_user );
-  }
-  return \@jobs;
+  return(OAR::IO::get_jobs_in_states_for_user($base, \@states, $user));
 }
 
 sub get_jobs_for_user_query {
@@ -115,6 +111,7 @@ sub get_jobs_for_user_query {
     my $limit = shift;
     my $offset = shift;
     my $array = shift;
+    my $ids = shift;
 	
     if (defined($state)) {
         my @states = split(/,/,$state);
@@ -127,7 +124,7 @@ sub get_jobs_for_user_query {
         $state = $statement;
     }
 
-    my %jobs =  OAR::IO::get_jobs_for_user_query($base,$from,$to,$state,$limit,$offset,$user,$array);
+    my %jobs =  OAR::IO::get_jobs_for_user_query($base,$from,$to,$state,$limit,$offset,$user,$array,$ids);
     return (\%jobs);
 }
 
@@ -137,6 +134,7 @@ sub count_jobs_for_user_query {
     my $to = shift;
     my $state = shift;
     my $array = shift;
+    my $ids=shift;
 	
 	if (defined($state)) {
 		my @states = split(/,/,$state);
@@ -149,12 +147,12 @@ sub count_jobs_for_user_query {
     $state = $statement;
 	}
 
-	my $total =  OAR::IO::count_jobs_for_user_query($base,$from,$to,$state,$user,undef,undef,$array);
+	my $total =  OAR::IO::count_jobs_for_user_query($base,$from,$to,$state,$user,undef,undef,$array,$ids);
 	return $total;
 }
 
 sub get_all_admission_rules() {
-	my @admission_rules = OAR::IO::list_admission_rules($base);
+	my @admission_rules = OAR::IO::list_admission_rules($base,undef);
 	return \@admission_rules;
 }
 
@@ -177,17 +175,6 @@ sub get_specific_admission_rule {
     return $rule;
 }
 
-sub add_admission_rule {
-	my $rule = shift;
-	my $id = OAR::IO::add_admission_rule($base,$rule);
-	return $id;
-}
-
-sub delete_specific_admission_rule {
-	my $rule_id = shift;
-	OAR::IO::delete_admission_rule($base,$rule_id);
-}
-
 sub get_duration($){
 # Converts a number of seconds in a human readable duration (years,days,hours,mins,secs)
     my $time=shift;
@@ -254,7 +241,7 @@ sub get_history($$){
     #print finished or running jobs
     my %jobs_history = OAR::IO::get_jobs_range_dates($base,$date_start,$date_stop);
     foreach my $i (keys(%jobs_history)){
-        my $types = OAR::IO::get_current_job_types($base,$i);
+        my $types = OAR::IO::get_job_types_hash($base,$i);
         if (!defined($job_gantt{$i}) || (defined($types->{besteffort}))){
             if (($jobs_history{$i}->{state} eq "Running") ||
                 ($jobs_history{$i}->{state} eq "toLaunch") ||
@@ -354,7 +341,6 @@ sub get_job_data($$){
     my @job_events;
     my %data_to_display;
     my $job_user;
-    my $job_cpuset_uid;
     my @job_dependencies;
     my @job_types = OAR::IO::get_job_types($dbh,$job_info->{job_id});
     my $cpuset_name;
@@ -379,9 +365,7 @@ sub get_job_data($$){
         @date_tmp = OAR::IO::get_gantt_job_start_time_visu($dbh,$job_info->{job_id});
         @job_events = OAR::IO::get_job_events($dbh,$job_info->{job_id});
         @job_dependencies = OAR::IO::get_current_job_dependencies($dbh,$job_info->{job_id});
-
-        $job_cpuset_uid = OAR::IO::get_job_cpuset_uid($dbh, $job_info->{assigned_moldable_job}, $Job_uid_resource_type, $Cpuset_field) if ((defined($Job_uid_resource_type)) and (defined($Cpuset_field)));
-        $job_user = OAR::Tools::format_job_user($job_info->{job_user},$job_info->{job_id},$job_cpuset_uid);
+        $job_user = $job_info->{job_user};
    
         #Get the job resource description to print -l option
         my $job_descriptions = OAR::IO::get_resources_data_structure_current_job($dbh,$job_info->{job_id});
@@ -420,7 +404,6 @@ sub get_job_data($$){
             name => $job_info->{job_name},
             owner => $job_info->{job_user},
             job_user => $job_user,
-            job_uid => $job_cpuset_uid,
             state => $job_info->{state},
             assigned_resources => \@nodes,
             assigned_network_address => \@node_hostnames,
@@ -536,4 +519,60 @@ sub get_job_state($) {
 	#return(\%default_job_infos);
 #}
 
+# Compact the array jobs
+# Replaces all jobs from an array by a single virtual job
+# having "N@array_id" as job_id, with N the number of jobs
+# into the array
+sub compact_arrays($){
+  my $jobs=shift;
+  my $array_job={};
+  my $newjobs=[];
+
+  # Parse all the jobs to search for arrays of more than 1 job
+  foreach my $job (@{$jobs}){
+      if (defined($array_job->{$job->{array_id}})) {
+        # New element of an array found, increasing the counter
+        $array_job->{$job->{array_id}}->{n} = $array_job->{$job->{array_id}}->{n} + 1;
+        $array_job->{$job->{array_id}}->{state}="NA";
+      }else{
+        $array_job->{$job->{array_id}}=$job;
+        $array_job->{$job->{array_id}}->{n}=1;
+      }
+  }
+  # Generate the new list of jobs
+  foreach my $job (@{$jobs}){
+    if ($array_job->{$job->{array_id}}->{n} > 1) {
+      # Do not output the jobs inside an array, but output the virtual job only once
+      if (!defined($array_job->{$job->{array_id}}->{already_printed})) {
+        $array_job->{$job->{array_id}}->{job_id}=$array_job->{$job->{array_id}}->{n}."@".$job->{array_id};
+        push (@{$newjobs}, $array_job->{$job->{array_id}});
+        $array_job->{$job->{array_id}}->{already_printed}=1;
+      }
+    }else{
+      # jobs not comming from an array are outputed normally
+      push (@{$newjobs}, $job);
+    }
+  }
+  return $newjobs;
+}
+
+# Use the current date in seconds from the EPOCH to determine the duration of a
+# running job (start_time>0)
+sub get_job_duration($$){
+    my $start_date = shift;
+    my $stop_date = shift;
+
+    if ($stop_date < $start_date){
+        if ($current_date < 0){
+            $current_date = OAR::IO::get_date($base);
+        }
+        $stop_date = $current_date;
+    }
+    my ($h,$m,$s) = (0,0,0);
+    if (($start_date > 0) and ($start_date < $stop_date)){
+        ($h,$m,$s) = OAR::IO::duration_to_hms($stop_date - $start_date);
+    }
+    return(sprintf("%i:%02i:%02i", $h,$m,$s));
+}
+
 1;
