@@ -1,8 +1,8 @@
 package OAR::Version;
 require Exporter;
 
-my $OARVersion = "2.5.2";
-my $OARName = "tzuhtzjo:m uy-u:xlaju:n pik";
+my $OARVersion = "2.5.4";
+my $OARName = "Froggy Summer";
 
 sub get_version(){
     return $OARVersion." (".$OARName.")";
