@@ -1,23 +1,24 @@
 -- Default admission rules for OAR 2
 -- $Id$
 
-DROP TABLE admission_rules;
-CREATE TABLE admission_rules (
-  id bigserial,
-  rule text NOT NULL,
-  PRIMARY KEY  (id)
-);
-
--- Default admission rules
-
 -- Specify the default value for queue parameter
-INSERT INTO admission_rules (rule) VALUES (E'if (not defined($queue_name)) {$queue_name="default";}');
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (1, 'YES', E'# Set default queue is no queue is set
+if (not defined($queue_name)) {$queue_name="default";}
+');
 
 -- Prevent root and oar to submit jobs.
-INSERT INTO admission_rules (rule) VALUES (E'die ("[ADMISSION RULE] root and oar users are not allowed to submit jobs.\\n") if ( $user eq "root" or $user eq "oar" );');
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (2, 'YES', E'# Prevent users oar and root to submit jobs
+# Note: do not change this unless you want to break oar !
+die ("[ADMISSION RULE] root and oar users are not allowed to submit jobs.\\n") if ( $user eq "root" or $user eq "oar" );
+');
+
+-- Avoid the jobs to go on resources in drain mode
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (3, 'YES', E'# Avoid the jobs to go on resources in drain mode
+$jobproperties_applied_after_validation = "drain=''NO''";
+');
 
 -- Avoid users except admin to go in the admin queue
-INSERT INTO admission_rules (rule) VALUES (E'
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (4, 'YES', E'# Restrict the admin queue to members of the admin group
 my $admin_group = "admin";
 if ($queue_name eq "admin") {
     my $members; 
@@ -30,7 +31,7 @@ if ($queue_name eq "admin") {
 ');
 
 -- Prevent the use of system properties
-INSERT INTO admission_rules (rule) VALUES (E'
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (5, 'YES', E'# Prevent users from using internal resource properties for oarsub requests 
 my @bad_resources = ("type","state","next_state","finaud_decision","next_finaud_decision","state_num","suspended_jobs","besteffort","deploy","expiry_date","desktop_computing","last_job_date","available_upto","scheduler_priority");
 foreach my $mold (@{$ref_resource_list}){
     foreach my $r (@{$mold->[0]}){
@@ -48,7 +49,7 @@ foreach my $mold (@{$ref_resource_list}){
 -- Force besteffort jobs to run in the besteffort queue
 -- Force job of the besteffort queue to be of the besteffort type
 -- Force besteffort jobs to run on nodes with the besteffort property
-INSERT INTO admission_rules (rule) VALUES (E'
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (6, 'YES', E'# Tie the besteffort queue, job type and resource property together
 if (grep(/^besteffort$/, @{$type_list}) and not $queue_name eq "besteffort"){
     $queue_name = "besteffort";
     print("[ADMISSION RULE] Automatically redirect in the besteffort queue\\n");
@@ -68,14 +69,14 @@ if (grep(/^besteffort$/, @{$type_list})){
 ');
 
 -- Verify if besteffort jobs are not reservations
-INSERT INTO admission_rules (rule) VALUES (E'
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (7, 'YES', E'# Prevent besteffort advance-reservation
 if ((grep(/^besteffort$/, @{$type_list})) and ($reservationField ne "None")){
     die("[ADMISSION RULE] Error: a job cannot both be of type besteffort and be a reservation.\\n");
 }
 ');
 
 -- Force deploy jobs to go on resources with the deploy property
-INSERT INTO admission_rules (rule) VALUES (E'
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (8, 'YES', E'# Tie the deploy job type and resource property together
 if (grep(/^deploy$/, @{$type_list})){
     if ($jobproperties ne ""){
         $jobproperties = "($jobproperties) AND deploy = ''YES''";
@@ -86,7 +87,7 @@ if (grep(/^deploy$/, @{$type_list})){
 ');
 
 -- Prevent deploy and allow_classic_ssh type jobs on none entire nodes
-INSERT INTO admission_rules (rule) VALUES (E'
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (9, 'YES', E'# Restrict allowed properties for deploy jobs to force requesting entire nodes
 my @bad_resources = ("core","cpu","resource_id",);
 if (grep(/^(deploy|allow_classic_ssh)$/, @{$type_list})){
     foreach my $mold (@{$ref_resource_list}){
@@ -104,7 +105,7 @@ if (grep(/^(deploy|allow_classic_ssh)$/, @{$type_list})){
 ');
 
 -- Force desktop_computing jobs to go on nodes with the desktop_computing property
-INSERT INTO admission_rules (rule) VALUES (E'
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (10, 'YES', E'# Tie desktop computing job type and resource property together
 if (grep(/^desktop_computing$/, @{$type_list})){
     print("[ADMISSION RULE] Added automatically desktop_computing resource constraints\\n");
     if ($jobproperties ne ""){
@@ -123,7 +124,7 @@ if (grep(/^desktop_computing$/, @{$type_list})){
 
 -- Limit the number of reservations that a user can do.
 -- (overrided on user basis using the file: ~oar/unlimited_reservation.users)
-INSERT INTO admission_rules (rule) VALUES (E'
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (11, 'YES', E'# Limit the number of advance reservations per user
 if ($reservationField eq "toSchedule") {
     my $unlimited=0;
     if (open(FILE, "< $ENV{HOME}/unlimited_reservation.users")) {
@@ -153,20 +154,20 @@ if ($reservationField eq "toSchedule") {
 }
 ');
 
--- How to perform actions if the user name is in a file
---INSERT INTO admission_rules (rule) VALUES (E'
---open(FILE, "/tmp/users.txt");
---while (($queue_name ne "admin") and ($_ = <FILE>)){
---    if ($_ =~ m/^\\s*$user\\s*$/m){
---        print("[ADMISSION RULE] Change assigned queue into admin\\n");
---        $queue_name = "admin";
---    }
---}
---close(FILE);
---');
+-- Example of how to perform actions given usernames stored in a file
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (12, 'NO', E'# Example of how to perform actions given usernames stored in a file
+open(FILE, "/tmp/users.txt");
+while (($queue_name ne "admin") and ($_ = <FILE>)){
+    if ($_ =~ m/^\\s*$user\\s*$/m){
+        print("[ADMISSION RULE] Change assigned queue to admin\\n");
+        $queue_name = "admin";
+    }
+}
+close(FILE);
+');
 
 -- Limit walltime for interactive jobs
-INSERT INTO admission_rules (rule) VALUES (E'
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (13, 'YES', E'# Limit the walltime for interactive jobs
 my $max_walltime = OAR::IO::sql_to_duration("12:00:00");
 if (($jobType eq "INTERACTIVE") and ($reservationField eq "None")){ 
     foreach my $mold (@{$ref_resource_list}){
@@ -179,7 +180,7 @@ if (($jobType eq "INTERACTIVE") and ($reservationField eq "None")){
 ');
 
 -- specify the default walltime if it is not specified
-INSERT INTO admission_rules (rule) VALUES (E'
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (14, 'YES', E'# Set the default walltime is not specified
 my $default_wall = OAR::IO::sql_to_duration("2:00:00");
 foreach my $mold (@{$ref_resource_list}){
     if (!defined($mold->[1])){
@@ -190,8 +191,8 @@ foreach my $mold (@{$ref_resource_list}){
 ');
 
 -- Check if types given by the user are right
-INSERT INTO admission_rules (rule) VALUES (E'
-my @types = ("container","inner","deploy","desktop_computing","besteffort","cosystem","idempotent","timesharing","allow_classic_ssh","token\\:xxx=yy");
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (15, 'YES', E'# Check if job types are valid
+my @types = ("container","inner","deploy","desktop_computing","besteffort","cosystem","noop","idempotent","timesharing","allow_classic_ssh","token\\:xxx=yy");
 foreach my $t (@{$type_list}){
     my $i = 0;
     while (($types[$i] ne $t) and ($i <= $#types)){
@@ -204,7 +205,7 @@ foreach my $t (@{$type_list}){
 ');
 
 -- If resource types are not specified, then we force them to default
-INSERT INTO admission_rules (rule) VALUES (E'
+INSERT INTO admission_rules (priority, enabled, rule) VALUES (16, 'YES', E'# Set resource type to default if not specified
 foreach my $mold (@{$ref_resource_list}){
     foreach my $r (@{$mold->[0]}){
         my $prop = $r->{property};
