@@ -211,12 +211,12 @@ sub nodelistByProperty {
           my %hashProp= %{$hashRessProp{$ress}};
           foreach my $p (keys %hashProp) {
                   my $hidden = undef;
-                  my $prop= $p."=".$hashProp{$p};
+                  my $prop= $p."=".(defined($hashProp{$p})?$hashProp{$p}:"?");
                   if($prop eq $property){
                     unless (defined($alreadyCounted{$node})){
                       $alreadyCounted{$node}= 1;
-                      #push @nodesSelected, $node->name;
-                      push @nodesSelected, $node->displayHTMLname;
+                      push @nodesSelected, $node->name;
+                      #push @nodesSelected, $node->displayHTMLname;
                     }
                   }
           }
@@ -389,6 +389,7 @@ sub htmlJobTable {
         $output .= $cgi->td({-align => "center"},"User");
         $output .= $cgi->td({-align => "center"},"State");
         $output .= $cgi->td({-align => "center"},"Queue");
+        $output .= $cgi->td({-align => "center"},"Name");
         #$output .= $cgi->td({-align => "center"},"NbNodes");
         #$output .= $cgi->td({-align => "center"},"NbCores");
         $output .= $cgi->td({-align => "center"},"wanted_resources");
